"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDbFactory = void 0;
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const services = __importStar(require("./"));
const creator = (dbFactoryConfig) => async () => {
    server_services_telemetry_1.Lumberjack.info(`Using ${dbFactoryConfig.name} Database.`);
    // Dynamically load the module
    const externalDbFactoryModule = await Promise.resolve().then(() => __importStar(require(dbFactoryConfig.modulePathOrName)));
    return new externalDbFactoryModule[dbFactoryConfig.dbFactoryConstructorName](dbFactoryConfig.dbConfig);
};
async function getDbFactory(config) {
    const dbFactoryConfig = config.get("db");
    // Default handling is Mongo
    if (!dbFactoryConfig || dbFactoryConfig.name === "mongo") {
        const mongoFactory = new services.MongoDbFactory(config.get("mongo"));
        return mongoFactory;
    }
    // We enable extensions loading only when the flag is set to true.
    // The idea behind the flag to have an additional security measure to avoid loading unwanted extensions.
    // @TODO: in the initial proposal it was part of the config file, but perhaps it should be an
    // environment variable instead ?
    if (config.get("loadExtensions") === true) {
        const dbExtensions = config.get("extensions:db") || [];
        const extension = dbExtensions.find((ext) => ext.name === dbFactoryConfig.name);
        if (extension === undefined) {
            throw new Error(`The database ${dbFactoryConfig.name} is not found.`);
        }
        // Instantiate the corresponding db factory
        const externalDbFactoryModule = creator(extension)();
        return externalDbFactoryModule;
    }
    throw new Error(`Couldn't load database: ${dbFactoryConfig.name}, perhaps you forgot to enable extensions loading?`);
}
exports.getDbFactory = getDbFactory;
//# sourceMappingURL=dbFactory.js.map