"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeltaManager = void 0;
const common_utils_1 = require("@fluidframework/common-utils");
const protocol_definitions_1 = require("@fluidframework/protocol-definitions");
const server_services_client_1 = require("@fluidframework/server-services-client");
const server_services_utils_1 = require("@fluidframework/server-services-utils");
const tenant_1 = require("./tenant");
/**
 * Manager to fetch deltas from Alfred using the internal URL.
 */
class DeltaManager {
    constructor(authEndpoint, internalAlfredUrl) {
        this.authEndpoint = authEndpoint;
        this.internalAlfredUrl = internalAlfredUrl;
    }
    async getDeltas(_collectionName, tenantId, documentId, from, to) {
        const baseUrl = `${this.internalAlfredUrl}`;
        const restWrapper = await this.getBasicRestWrapper(tenantId, documentId, baseUrl);
        const resultP = restWrapper.get(`/deltas/${tenantId}/${documentId}`, { from, to });
        return resultP;
    }
    async getDeltasFromStorage(collectionName, tenantId, documentId, fromTerm, toTerm, fromSeq, toSeq) {
        throw new Error("Method not implemented.");
    }
    async getDeltasFromSummaryAndStorage(collectionName, tenantId, documentId, from, to) {
        throw new Error("Method not implemented.");
    }
    async getKey(tenantId, includeDisabledTenant = false) {
        const tenantManager = new tenant_1.TenantManager(this.authEndpoint, "");
        const keyP = await tenantManager.getKey(tenantId, includeDisabledTenant);
        return keyP;
    }
    async getBasicRestWrapper(tenantId, documentId, baseUrl) {
        const key = await this.getKey(tenantId);
        const defaultQueryString = {
            token: (0, common_utils_1.fromUtf8ToBase64)(`${tenantId}`),
        };
        const getDefaultHeaders = () => {
            const token = { jwt: (0, server_services_utils_1.generateToken)(tenantId, documentId, key, [protocol_definitions_1.ScopeType.DocRead]) };
            return ({
                Authorization: `Basic ${token.jwt}`,
            });
        };
        const restWrapper = new server_services_client_1.BasicRestWrapper(baseUrl, defaultQueryString, undefined, undefined, getDefaultHeaders(), undefined, undefined, getDefaultHeaders, server_services_utils_1.getCorrelationId);
        return restWrapper;
    }
}
exports.DeltaManager = DeltaManager;
//# sourceMappingURL=deltaManager.js.map