"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMessageReceiver = void 0;
const events_1 = require("events");
const amqp = __importStar(require("amqplib"));
const winston = __importStar(require("winston"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
class RabbitmqReceiver {
    constructor(rabbitmqConfig, taskQueueName) {
        this.rabbitmqConfig = rabbitmqConfig;
        this.taskQueueName = taskQueueName;
        this.events = new events_1.EventEmitter();
        this.rabbitmqConnectionString = this.rabbitmqConfig.connectionString;
    }
    async initialize() {
        this.connection = await amqp.connect(this.rabbitmqConnectionString);
        this.channel = await this.connection.createChannel();
        await this.channel.assertQueue(this.taskQueueName, { durable: true });
        winston.info(`Rabbitmq task channel ready to receive!`);
        server_services_telemetry_1.Lumberjack.info(`Rabbitmq task channel ready to receive!`);
        // We don't need to ack the task messages since they will be part of next help message if unacked.
        // TODO: Reject messages and make sure the sender knows.
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        this.channel.consume(this.taskQueueName, (msgBuffer) => {
            const msgString = msgBuffer.content.toString();
            const msg = JSON.parse(msgString);
            this.events.emit("message", msg);
        }, { noAck: true });
        this.connection.on("error", (error) => {
            this.events.emit("error", error);
        });
    }
    on(event, listener) {
        this.events.on(event, listener);
        return this;
    }
    async close() {
        const closeChannelP = this.channel.close();
        const closeConnectionP = this.connection.close();
        await Promise.all([closeChannelP, closeConnectionP]);
    }
}
// Factory to switch between different message receiver.
function createMessageReceiver(rabbitmqConfig, queueName) {
    return new RabbitmqReceiver(rabbitmqConfig, queueName);
}
exports.createMessageReceiver = createMessageReceiver;
//# sourceMappingURL=messageReceiver.js.map