"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMessageSender = void 0;
const events_1 = require("events");
const server_services_core_1 = require("@fluidframework/server-services-core");
const amqp = __importStar(require("amqplib"));
const winston = __importStar(require("winston"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
class RabbitmqTaskSender {
    constructor(rabbitmqConfig, config) {
        this.events = new events_1.EventEmitter();
        this.rabbitmqConnectionString = rabbitmqConfig.connectionString;
        this.taskQueues = config.queues;
    }
    async initialize() {
        this.connection = await amqp.connect(this.rabbitmqConnectionString);
        this.channel = await this.connection.createChannel();
        // Assert task queues.
        const queuePromises = [];
        for (const queue of this.taskQueues) {
            queuePromises.push(this.channel.assertQueue(queue, { durable: true }));
        }
        await Promise.all(queuePromises);
        winston.info(`Rabbitmq task queues ready to produce!`);
        server_services_telemetry_1.Lumberjack.info(`Rabbitmq task queues ready to produce!`);
        this.connection.on("error", (error) => {
            this.events.emit("error", error);
        });
    }
    sendTask(queueName, message) {
        this.channel.sendToQueue(queueName, Buffer.from(JSON.stringify(message)), { persistent: false });
    }
    on(event, listener) {
        this.events.on(event, listener);
        return this;
    }
    async close() {
        const closeChannelP = this.channel.close();
        const closeConnectionP = this.connection.close();
        await Promise.all([closeChannelP, closeConnectionP]);
    }
}
// Factory to switch between specific message sender implementations. Returns a dummy implementation
// if rabbitmq configs are not provided.
function createMessageSender(rabbitmqConfig, config) {
    // eslint-disable-next-line @typescript-eslint/prefer-optional-chain
    return rabbitmqConfig && rabbitmqConfig.connectionString
        ? new RabbitmqTaskSender(rabbitmqConfig, config)
        : new server_services_core_1.EmptyTaskMessageSender();
}
exports.createMessageSender = createMessageSender;
//# sourceMappingURL=messageSender.js.map