/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import * as core from "@fluidframework/server-services-core";
import { AggregationCursor, Collection, MongoClient } from "mongodb";
import { MongoErrorRetryAnalyzer } from "./mongoExceptionRetryRules";
export declare class MongoCollection<T> implements core.ICollection<T>, core.IRetryable {
    private readonly collection;
    readonly retryEnabled: boolean;
    private readonly telemetryEnabled;
    private readonly mongoErrorRetryAnalyzer;
    constructor(collection: Collection<T>, retryEnabled: boolean, telemetryEnabled: boolean, mongoErrorRetryAnalyzer: MongoErrorRetryAnalyzer);
    aggregate(pipeline: any, options?: any): Promise<AggregationCursor<T>>;
    find(query: object, sort: any, limit?: number, skip?: number): Promise<T[]>;
    findOne(query: object): Promise<T>;
    findAll(): Promise<T[]>;
    update(filter: object, set: any, addToSet: any): Promise<void>;
    updateMany(filter: object, set: any, addToSet: any): Promise<void>;
    upsert(filter: object, set: any, addToSet: any): Promise<void>;
    distinct(key: any, query: any): Promise<any>;
    deleteOne(filter: any): Promise<any>;
    deleteMany(filter: any): Promise<any>;
    insertOne(value: T): Promise<any>;
    insertMany(values: T[], ordered: boolean): Promise<void>;
    createIndex(index: any, unique: boolean): Promise<void>;
    createTTLIndex(index: any, expireAfterSeconds?: number): Promise<void>;
    findOrCreate(query: any, value: T): Promise<{
        value: T;
        existing: boolean;
    }>;
    findAndUpdate(query: any, value: T): Promise<{
        value: T;
        existing: boolean;
    }>;
    private updateCore;
    private updateManyCore;
    private requestWithRetry;
    private getTelemetryPropertiesFromQuery;
}
export declare class MongoDb implements core.IDb {
    private readonly client;
    private readonly retryEnabled;
    private readonly telemetryEnabled;
    private readonly mongoErrorRetryAnalyzer;
    constructor(client: MongoClient, retryEnabled: boolean, telemetryEnabled: boolean, mongoErrorRetryAnalyzer: MongoErrorRetryAnalyzer);
    close(): Promise<void>;
    on(event: core.IDbEvents, listener: (...args: any[]) => void): void;
    collection<T>(name: string): core.ICollection<T>;
    dropCollection(name: string): Promise<boolean>;
}
interface IMongoDBConfig {
    operationsDbEndpoint: string;
    bufferMaxEntries: number | undefined;
    globalDbEndpoint?: string;
    globalDbEnabled?: boolean;
    connectionPoolMinSize?: number;
    connectionPoolMaxSize?: number;
    facadeLevelRetry?: boolean;
    facadeLevelTelemetry?: boolean;
    facadeLevelRetryRuleOverride?: any;
}
export declare class MongoDbFactory implements core.IDbFactory {
    private readonly operationsDbEndpoint;
    private readonly bufferMaxEntries?;
    private readonly globalDbEndpoint?;
    private readonly connectionPoolMinSize?;
    private readonly connectionPoolMaxSize?;
    private readonly retryEnabled;
    private readonly telemetryEnabled;
    private readonly retryRuleOverride;
    constructor(config: IMongoDBConfig);
    connect(global?: boolean): Promise<core.IDb>;
}
export {};
//# sourceMappingURL=mongodb.d.ts.map