"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeCodeLoader = exports.NodeAllowList = void 0;
// TODO: Move this in a separate package.
// import { ICodeLoader, ICodeAllowList, IFluidCodeDetails } from "@fluidframework/container-definitions";
const child_process_1 = require("child_process");
const fs = __importStar(require("fs"));
const util_1 = require("util");
const winston = __importStar(require("winston"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
const asyncExec = (0, util_1.promisify)(child_process_1.exec);
// A sentinel file to indicate install completion.
const signalFileName = "dummy";
class NodeAllowList {
    constructor() {
    }
    async testSource(source) {
        return Promise.resolve(true);
    }
}
exports.NodeAllowList = NodeAllowList;
class NodeCodeLoader {
    constructor(packageDirectory, waitTimeoutMSec, allowList) {
        this.packageDirectory = packageDirectory;
        this.waitTimeoutMSec = waitTimeoutMSec;
        this.allowList = allowList;
    }
    async load(pkg) {
        if (await this.allowList.testSource(pkg)) {
            let packageName = "";
            packageName = typeof pkg.package === "string"
                ? pkg.package
                : `${pkg.package.name}@${pkg.package.version}`;
            const codeEntrypoint = await this.installOrWaitForPackages(packageName);
            const entry = Promise.resolve().then(() => __importStar(require(codeEntrypoint)));
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return entry;
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-floating-promises
            Promise.reject(new Error("Invalid Package"));
        }
    }
    async installOrWaitForPackages(pkg) {
        const dataStores = pkg.match(/(.*)\/(.*)@(.*)/);
        if (!dataStores) {
            return Promise.reject(new Error("Invalid package"));
        }
        const [, scope, name] = dataStores;
        const packageDirectory = `${this.packageDirectory}/${pkg}`;
        const signalPath = `${packageDirectory}/${signalFileName}`;
        const codeEntrypoint = `${packageDirectory}/node_modules/${scope}/${name}`;
        if (!fs.existsSync(packageDirectory)) {
            // Our node version (8.15) does not support recursive directory creation.
            // Need to create each subdirectories manually.
            const dirs = pkg.split("/");
            if (!fs.existsSync(`${this.packageDirectory}/${dirs[0]}`)) {
                fs.mkdirSync(`${this.packageDirectory}/${dirs[0]}`);
            }
            if (!fs.existsSync(packageDirectory)) {
                fs.mkdirSync(packageDirectory);
            }
            // Copy over template package.json and .npmrc file to the directory where
            // npm install will be executed.
            fs.copyFileSync(`${this.packageDirectory}/package.json`, `${packageDirectory}/package.json`);
            fs.copyFileSync(`${this.packageDirectory}/.npmrc`, `${packageDirectory}/.npmrc`);
            // Run npm install
            await asyncExec(`npm install ${pkg}`, { cwd: packageDirectory });
            // Write dummy signal file to indicate package installation success.
            fs.closeSync(fs.openSync(signalPath, "w"));
            winston.info(`Installed ${pkg} in ${packageDirectory} directory`);
            server_services_telemetry_1.Lumberjack.info(`Installed ${pkg} in ${packageDirectory} directory`);
            return codeEntrypoint;
        }
        else {
            await this.waitForPackageFiles(packageDirectory, signalFileName, this.waitTimeoutMSec);
            winston.info(`Package ${pkg} is already installed`);
            server_services_telemetry_1.Lumberjack.info(`Package ${pkg} is already installed`);
            return codeEntrypoint;
        }
    }
    // A timeout based watcher that looks for dummy file creation.
    async waitForPackageFiles(targetDirectory, fileName, waitTimeout) {
        return new Promise((resolve, reject) => {
            const watcher = fs.watch(targetDirectory, (eventType, newFileName) => {
                if (eventType === "rename" && newFileName === fileName) {
                    clearTimeout(waitTimer);
                    watcher.close();
                    resolve();
                }
            });
            const waitTimer = setTimeout(() => {
                watcher.close();
                clearTimeout(waitTimer);
                reject(new Error(`${fileName} in ${targetDirectory} was not generated within ${waitTimeout} msecs`));
            }, waitTimeout);
            if (fs.existsSync(`${targetDirectory}/${fileName}`)) {
                clearTimeout(waitTimer);
                watcher.close();
                resolve();
            }
        });
    }
}
exports.NodeCodeLoader = NodeCodeLoader;
//# sourceMappingURL=nodeCodeLoader.js.map