"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisCache = void 0;
const winston = __importStar(require("winston"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
/**
 * Redis based cache client
 */
class RedisCache {
    constructor(client, parameters) {
        this.client = client;
        this.expireAfterSeconds = 60 * 60 * 24;
        this.prefix = "page";
        if (parameters === null || parameters === void 0 ? void 0 : parameters.expireAfterSeconds) {
            this.expireAfterSeconds = parameters.expireAfterSeconds;
        }
        if (parameters === null || parameters === void 0 ? void 0 : parameters.prefix) {
            this.prefix = parameters.prefix;
        }
        client.on("error", (err) => {
            winston.error("Error with Redis:", err);
            server_services_telemetry_1.Lumberjack.error("Error with Redis", undefined, err);
        });
    }
    async delete(key) {
        try {
            await this.client.del(this.getKey(key));
            return true;
        }
        catch (error) {
            server_services_telemetry_1.Lumberjack.error(`Error deleting from cache.`, undefined, error);
            return false;
        }
    }
    async get(key) {
        return this.client.get(this.getKey(key));
    }
    async set(key, value, expireAfterSeconds) {
        const result = await this.client.set(this.getKey(key), value, "EX", expireAfterSeconds !== null && expireAfterSeconds !== void 0 ? expireAfterSeconds : this.expireAfterSeconds);
        if (result !== "OK") {
            return Promise.reject(result);
        }
    }
    async incr(key) {
        try {
            return this.client.incr(key);
        }
        catch (error) {
            server_services_telemetry_1.Lumberjack.error(`Error while incrementing counter for ${key} in redis.`, undefined, error);
            return Promise.reject(error);
        }
    }
    async decr(key) {
        try {
            return this.client.decr(key);
        }
        catch (error) {
            server_services_telemetry_1.Lumberjack.error(`Error while decrementing counter for ${key} in redis.`, undefined, error);
            return Promise.reject(error);
        }
    }
    /**
     * Translates the input key to the one we will actually store in redis
     */
    getKey(key) {
        return `${this.prefix}:${key}`;
    }
}
exports.RedisCache = RedisCache;
//# sourceMappingURL=redis.js.map