"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientManager = void 0;
const server_services_utils_1 = require("@fluidframework/server-services-utils");
const winston = __importStar(require("winston"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
// Manages the set of connected clients in redis hashes with an expiry of 'expireAfterSeconds'.
class ClientManager {
    constructor(client, parameters) {
        this.client = client;
        this.expireAfterSeconds = 60 * 60 * 24;
        this.prefix = "client";
        if (parameters === null || parameters === void 0 ? void 0 : parameters.expireAfterSeconds) {
            this.expireAfterSeconds = parameters.expireAfterSeconds;
        }
        if (parameters === null || parameters === void 0 ? void 0 : parameters.prefix) {
            this.prefix = parameters.prefix;
        }
        client.on("error", (error) => {
            winston.error("Client Manager Redis Error:", error);
            server_services_telemetry_1.Lumberjack.error("Client Manager Redis Error", undefined, error);
        });
    }
    async addClient(tenantId, documentId, clientId, details) {
        const key = this.getKey(tenantId, documentId);
        const data = { [clientId]: JSON.stringify(details) };
        return (0, server_services_utils_1.executeRedisMultiWithHmsetExpire)(this.client, key, data, this.expireAfterSeconds);
    }
    async removeClient(tenantId, documentId, clientId) {
        await this.client.hdel(this.getKey(tenantId, documentId), clientId);
    }
    async getClients(tenantId, documentId) {
        const dbClients = await this.client.hgetall(this.getKey(tenantId, documentId));
        const clients = [];
        if (dbClients) {
            for (const clientId of Object.keys(dbClients)) {
                clients.push({
                    clientId,
                    client: JSON.parse(dbClients[clientId]),
                });
            }
        }
        return clients;
    }
    /**
     * Returns all clients currently connected including a keep alive time.
     * Should be used with delis read only client functionality.
     */
    async getSequencedClients(tenantId, documentId) {
        throw new Error("Not implemented");
    }
    /**
     * Called when the expiration time of clients should be extended.
     * @param clientTimeout - Amount of time in milliseconds to add to the clients expiration time.
     */
    async extendSequencedClients(tenantId, documentId, clients, clientTimeout) {
        throw new Error("Not implemented");
    }
    getKey(tenantId, documentId) {
        return `${this.prefix}:${tenantId}:${documentId}`;
    }
}
exports.ClientManager = ClientManager;
//# sourceMappingURL=redisClientManager.js.map