"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisThrottleAndUsageStorageManager = void 0;
const server_services_utils_1 = require("@fluidframework/server-services-utils");
const winston = __importStar(require("winston"));
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
/**
 * Manages storage of throttling metrics and usage data in redis.
 */
class RedisThrottleAndUsageStorageManager {
    constructor(client, parameters) {
        this.client = client;
        this.expireAfterSeconds = 60 * 60 * 24;
        this.prefix = "throttle";
        if (parameters === null || parameters === void 0 ? void 0 : parameters.expireAfterSeconds) {
            this.expireAfterSeconds = parameters.expireAfterSeconds;
        }
        if (parameters === null || parameters === void 0 ? void 0 : parameters.prefix) {
            this.prefix = parameters.prefix;
        }
        client.on("error", (error) => {
            winston.error("Throttle Manager Redis Error:", error);
            server_services_telemetry_1.Lumberjack.error("Throttle Manager Redis Error", { [server_services_telemetry_1.CommonProperties.telemetryGroupName]: "throttling" }, error);
        });
    }
    async setThrottlingMetric(id, throttlingMetric) {
        const throttlingKey = this.getKey(id);
        return (0, server_services_utils_1.executeRedisMultiWithHmsetExpire)(this.client, throttlingKey, throttlingMetric, this.expireAfterSeconds);
    }
    async setThrottlingMetricAndUsageData(throttlingId, throttlingMetric, usageStorageId, usageData) {
        const throttlingKey = this.getKey(throttlingId);
        const usageDataString = JSON.stringify(usageData);
        server_services_telemetry_1.Lumberjack.info(`Pushing usage data - id: ${usageStorageId}, data: ${usageDataString}}`, {
            [server_services_telemetry_1.BaseTelemetryProperties.tenantId]: usageData.tenantId,
            [server_services_telemetry_1.BaseTelemetryProperties.documentId]: usageData.documentId,
            [server_services_telemetry_1.CommonProperties.clientId]: usageData.clientId,
        });
        return (0, server_services_utils_1.executeRedisMultiWithHmsetExpireAndLpush)(this.client, throttlingKey, throttlingMetric, usageStorageId, usageDataString, this.expireAfterSeconds);
    }
    async getThrottlingMetric(id) {
        const throttlingMetric = await this.client.hgetall(this.getKey(id));
        if (Object.keys(throttlingMetric).length === 0) {
            return undefined;
        }
        // All values retrieved from Redis are strings, so they must be parsed
        return {
            count: Number.parseInt(throttlingMetric.count, 10),
            lastCoolDownAt: Number.parseInt(throttlingMetric.lastCoolDownAt, 10),
            throttleStatus: throttlingMetric.throttleStatus === "true",
            throttleReason: throttlingMetric.throttleReason,
            retryAfterInMs: Number.parseInt(throttlingMetric.retryAfterInMs, 10),
        };
    }
    async setUsageData(id, usageData) {
        const usageDataString = JSON.stringify(usageData);
        server_services_telemetry_1.Lumberjack.info(`Pushing usage data - id: ${id}, data: ${usageDataString}}`, {
            [server_services_telemetry_1.BaseTelemetryProperties.tenantId]: usageData.tenantId,
            [server_services_telemetry_1.BaseTelemetryProperties.documentId]: usageData.documentId,
            [server_services_telemetry_1.CommonProperties.clientId]: usageData.clientId,
        });
        await this.client.lpush(id, usageDataString);
    }
    async getUsageData(id) {
        const usageDataString = await this.client.rpop(id);
        if (usageDataString) {
            return JSON.parse(usageDataString);
        }
        return undefined;
    }
    getKey(id) {
        return `${this.prefix}:${id}`;
    }
}
exports.RedisThrottleAndUsageStorageManager = RedisThrottleAndUsageStorageManager;
//# sourceMappingURL=redisThrottleAndUsageStorageManager.js.map