"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketIoRedisPublisher = exports.SocketIoRedisTopic = void 0;
const events_1 = require("events");
const util = __importStar(require("util"));
const ioredis_1 = __importDefault(require("ioredis"));
const redis_emitter_1 = require("@socket.io/redis-emitter");
class SocketIoRedisTopic {
    constructor(topic) {
        this.topic = topic;
    }
    emit(event, ...args) {
        this.topic.emit(event, ...args);
    }
}
exports.SocketIoRedisTopic = SocketIoRedisTopic;
class SocketIoRedisPublisher {
    constructor(options) {
        this.events = new events_1.EventEmitter();
        this.redisClient = new ioredis_1.default(options);
        this.io = new redis_emitter_1.Emitter(this.redisClient);
        this.redisClient.on("error", (error) => {
            this.events.emit("error", error);
        });
    }
    on(event, listener) {
        this.events.on(event, listener);
    }
    to(topic) {
        // NOTE - socket.io-emitter maintains local state during an emit request so we cannot cache the result of
        // doing a to, etc...
        return new SocketIoRedisTopic(this.io.to(topic));
    }
    async emit(topic, event, ...args) {
        this.io.to(topic).emit(event, ...args);
    }
    // eslint-disable-next-line @typescript-eslint/promise-function-async
    close() {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return util.promisify(((callback) => this.redisClient.quit(callback)))();
    }
}
exports.SocketIoRedisPublisher = SocketIoRedisPublisher;
//# sourceMappingURL=socketIoRedisPublisher.js.map